# Copyright (c) 2022 Cohere Inc. and its affiliates.
#
# Licensed under the MIT License (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License in the LICENSE file at the top
# level of this repository.

from conversant.prompts.chat_prompt import ChatPrompt
from conversant.prompts.prompt import Prompt
from conversant.prompts.rewrite_prompt import RewritePrompt

__all__ = ["Prompt", "ChatPrompt", "RewritePrompt"]



# Copyright (c) 2022 Cohere Inc. and its affiliates.
#
# Licensed under the MIT License (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License in the LICENSE file at the top
# level of this repository.


import json
import os
import re
from typing import List

import emoji
import emojificate
import streamlit as st
from emojificate.filter import emojificate as emojificate_fn

from conversant.prompt_chatbot import PERSONA_MODEL_DIRECTORY, PromptChatbot
from conversant.prompts.chat_prompt import ChatPrompt

emojificate.filter.TWITTER_CDN = "https://cdnjs.cloudflare.com/ajax/libs/twemoji/14.0.2"


class ParrotChatbot(PromptChatbot):
    """Mock chat function; real use-cases should import
    this functionality via a class that inherits from conversant.chatbot.Chatbot

    This bot simply states the user's query.
    """

    def __init__(self):
        super().__init__(
            client=None,
            prompt=ChatPrompt(
                preamble=(
                    "The Parrot Bot repeats back whatever is said to it "
                    "without using Cohere's large language models."
                ),
                fields=["user", "bot"],
                headers={
                    "user": "User",
                    "bot": "Parrot Bot",
                },
                example_separator="",
                examples=[],
            ),
        )

    def reply(self, query: str) -> str:
        """Replies to a user by stating their query.

        Args:
            query (str): A user's text chat query

        Returns:
            str: a mock reply that repeats the user's query.
        """
        current_prompt = self.get_current_prompt(query)
        self.chat_history.append(self.prompt.create_interaction(query, query))
        self.prompt_history.append(current_prompt)
        return query


@st.cache
def get_twemoji_url_from_shortcode(shortcode: str) -> str:

    """Converts an emoji shortcode to its corresponding Twemoji URL.

    Args:
        shortcode (str): Emoji shortcode.

    Returns:
        str: The string that is the Twemoji URL corresponding to the emoji.
    """
    # Emojize returns the unicode representation of that emoji from its shortcode.
    unicode = emoji.emojize(shortcode, language="alias")
    # Emojificate returns html <img /> tag.
    img_html_tag = emojificate_fn(unicode)
    # Find the URL from the html tag.
    url = re.findall('src="(.*?)"', img_html_tag, re.DOTALL)[0]
    return url


@st.cache(allow_output_mutation=True)
def get_persona_options(persona_directory: str = PERSONA_MODEL_DIRECTORY) -> List[str]:
    """Initializes a list of personas.

    Each persona is a directory in persona_directory, each with its
    config.json file. The mock parrot persona is also included for testing
    purposes.

    Args:
        persona_directory (str): Directory where persona folders containing config
        files are stored. Defaults to PERSONA_MODEL_DIRECTORY.

    Returns:
        List[str]: A list of persona names.
    """
    # Initialize the list of personas for Streamlit
    persona_names = os.listdir(persona_directory)
    persona_names_maybe_with_emojis = []
    for persona_name in persona_names:
        persona_path = os.path.join(persona_directory, persona_name, "config.json")
        with open(persona_path) as f:
            persona = json.load(f)
            avatar = (
                emoji.emojize(persona["chatbot_config"]["avatar"], language="alias")
                if "avatar" in persona["chatbot_config"]
                else ""
            )
            persona_names_maybe_with_emojis.append(
                f"{avatar} {persona_name}"
            ) if emoji.is_emoji(avatar) else persona_names_maybe_with_emojis.append(
                persona_name
            )
    persona_options = [""] + sorted(
        persona_names_maybe_with_emojis, reverse=True
    )  # + ["parrot"]
    return persona_options


def style_using_css(style: str) -> None:
    """Utility function to inject CSS style into the Streamlit DOM.

    Args:
        style (str): String representation of CSS style. Assumes it is well-formed.
    """
    st.markdown(f"<style>{style}</style>", unsafe_allow_html=True)


def escape_string(string: str) -> str:
    """Utility function to add '\' to escape sequences in a string.

    Args:
        string (str): Unescaped string.

    Returns:
        str: Escaped string
    """
    return string.encode("unicode_escape").decode("raw_unicode_escape")


def unescape_string(string: str) -> str:
    """Utility function to remove '\' from escape sequences in a string.

    Args:
        string (str): Escaped string.

    Returns:
        str: Unescaped string
    """
    return string.encode("raw_unicode_escape").decode("unicode_escape")



# Copyright (c) 2022 Cohere Inc. and its affiliates.
#
# Licensed under the MIT License (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License in the LICENSE file at the top
# level of this repository.


from collections import defaultdict

import streamlit as st
from streamlit_ace import st_ace
from streamlit_talk import message as st_message

from conversant.demo import utils


def draw_chat_history() -> None:
    """Renders the chat history in Streamlit.

    The messages are rendered using streamlit-chat, a custom Streamlit component
    for a chatbot UI.
    Reference: https://github.com/AI-Yash/st-chat
    """
    # The chat history is iterated in reverse order to ensure that recent messages
    # displayed are anchored at the bottom of the Streamlit demo.
    for i, turn in enumerate(reversed(st.session_state.bot.chat_history)):

        # If we are at the first conversation turn, we remove the
        # injected user utterance of "Hello" from displaying.
        if i == len(st.session_state.bot.chat_history) - 1:
            if "bot" in turn:
                st_message(
                    turn["bot"],
                    key=f"{i}_bot",
                    avatar_style=st.session_state.bot_avatar,
                )
        else:
            if "bot" in turn:
                st_message(
                    turn["bot"],
                    key=f"{i}_bot",
                    avatar_style=st.session_state.bot_avatar,
                )
            if "user" in turn:
                st_message(
                    turn["user"],
                    is_user=True,
                    key=f"{i}_user",
                    avatar_style=st.session_state.user_avatar,
                )


def draw_disclaimer() -> None:
    """Adds a disclaimer about the personas in this demo."""
    if st.session_state.persona != "parrot":
        st.write(
            "_Each persona is powered by [Cohere](https://cohere.com)'s large language "
            "models, and these examples are meant purely for demonstrative purposes. "
            "These personas are works of fiction, are not factually grounded, and "
            "should not be taken too seriously!_"
        )
    else:
        st.write(
            "_The Parrot persona does not make use of [Cohere](https://cohere.com)'s "
            "large language models. Instead, it repeats back whatever message it "
            "receives._"
        )


def draw_chatbot_config_form() -> None:
    """Adds widgets to edit the chatbot config."""
    config = st.session_state.snapshot_chatbot_config
    max_context_examples = st.slider(
        label="max_context_examples",
        min_value=0,
        max_value=20,
        value=config["max_context_examples"],
        help="The number of interactions to keep as context for the chatbot.",
    )
    st.session_state.bot.configure_chatbot(
        {"max_context_examples": max_context_examples}
    )


def draw_client_config_form() -> None:
    """Adds widgets to edit the client config."""
    st.write(
        "For more information on these parameters, see "
        "https://docs.cohere.ai/generate-reference"
    )
    config = st.session_state.snapshot_client_config
    model_options = ["", "small", "medium", "large", "xlarge"]
    model = st.selectbox(
        label="model",
        options=model_options,
        index=model_options.index(config["model"])
        if config["model"] in model_options
        else 0,
        help="The size of the Cohere model used to generate with.",
    )
    model_id_override = st.text_input(
        label="model ID override",
        value=model if model else config["model"],
        help=(
            "The full ID of a custom model. See "
            "https://docs.cohere.ai/generate-reference#model-optional for more details."
        ),
    )
    if model != model_id_override:
        st.warning(
            "WARNING: This demo does not validate that the model ID used for override "
            "is valid.",
        )
    max_tokens = st.number_input(
        label="max_tokens",
        value=config["max_tokens"],
        help="The number of tokens to predict per response.",
    )
    temperature = st.slider(
        label="temperature",
        min_value=0.0,
        max_value=5.0,
        value=config["temperature"],
        help=(
            "The degree of randomness for the response. Large temperature values may "
            "yield overly random results!"
        ),
    )
    frequency_penalty = st.slider(
        label="frequency_penalty",
        min_value=0.0,
        max_value=1.0,
        value=config["frequency_penalty"],
        help=(
            "Penalty to reduce repetitiveness of generated tokens, weighted by their "
            "frequency. Large penalty values may yield strange results!"
        ),
    )
    presence_penalty = st.slider(
        label="presence_penalty",
        min_value=0.0,
        max_value=1.0,
        value=config["presence_penalty"],
        help=(
            "Penalty to reduce repetitiveness of generated tokens, weighted equally "
            "to all present tokens. Large penalty values may yield strange results!"
        ),
    )
    # This allows the user to add their own stop sequences to a multiselect form
    # below.
    if "current_stop_sequences" not in st.session_state:
        st.session_state.current_stop_sequences = [
            utils.escape_string(stop_seq) for stop_seq in config["stop_sequences"]
        ]
    new_stop_seq = st.text_input(
        label="add new stop sequence",
        help="Add a stop sequence to the selection below.",
    )
    if (
        new_stop_seq != ""
        and new_stop_seq not in st.session_state.current_stop_sequences
    ):
        st.session_state.current_stop_sequences.append(new_stop_seq)
    # Use the list of stop sequences in the session state, including any user added ones
    # as the defaults for a multiselect form.
    st.multiselect(
        label="stop_sequences",
        options=st.session_state.current_stop_sequences,
        default=st.session_state.current_stop_sequences,
        key="selected_stop_sequences",
        help=(
            "The generated response will be cut off at the first instance of any of "
            "these stop sequences."
        ),
    )

    st.session_state.bot.configure_client(
        {
            "model": model_id_override,
            "max_tokens": max_tokens,
            "temperature": temperature,
            "frequency_penalty": frequency_penalty,
            "presence_penalty": presence_penalty,
            "stop_sequences": [
                utils.unescape_string(stop_seq)
                for stop_seq in st.session_state.selected_stop_sequences
            ],  # Stop sequences need to be unescaped e.g. from \\n to \n
        }
    )


def draw_prompt_form(disabled: bool = False) -> None:
    """Adds a form for configuring the prompt through its fields.

    The form is rendered as disabled when we only need to show the non-editable values
    of a prompt. This is used when the JSON editor is active.

    Args:
        disabled (bool): Whether or not the form should be rendered as disabled.
    """
    # Batches elements together as a form with a common submit button.
    with st.form("prompt_form"):
        # When the form is disabled, each time it is rendered its values need to be
        # taken from the current prompt config. Otherwise, its values should be taken
        # from the snapshot of the prompt config whenever it is first rendered.
        config = (
            defaultdict(str, st.session_state.bot.prompt.to_dict())
            if disabled
            else defaultdict(str, st.session_state.snapshot_prompt_config)
        )
        # We need to be careful about indexing into the dictionaries here
        # because when editing the prompt JSON, keys can end up malformed.
        default_preamble = config["preamble"]
        default_example_separator = config["example_separator"]
        default_user_name = (
            config["headers"]["user"] if "user" in config["headers"] else ""
        )
        default_bot_name = (
            config["headers"]["bot"] if "bot" in config["headers"] else ""
        )
        # This is where we create the text areas for the form.
        preamble = st.text_area(
            label="preamble",
            disabled=disabled,
            value=utils.escape_string(
                default_preamble
            ),  # Display chars like \n in the text area by escaping them to \\n
            help=(
                "A string that directs the chatbot to behae in certain ways by "
                "describing its function and characteristics (i.e. a description of "
                "a bot's persona). Accepts escape sequences like \\n."
            ),
        )
        example_separator = st.text_input(
            label="example_separator",
            disabled=disabled,
            value=utils.escape_string(
                default_example_separator
            ),  # Display chars like \n in the text area by escaping them to \\n
            help="A separator for each example. Accepts escape sequences like \\n.",
        )
        user_name = st.text_input(
            label="user",
            disabled=disabled,
            value=utils.escape_string(
                default_user_name
            ),  # Display chars like \n in the text area by escaping them to \\n
            help="The name of the user. Defaults to 'User'.",
        )
        bot_name = st.text_input(
            label="bot",
            disabled=disabled,
            value=utils.escape_string(
                default_bot_name
            ),  # Display chars like \n in the text area by escaping them to \\n
            help="The name of the chatbot.",
        )
        # Because prompt examples have a more complex structure, it is not very user
        # friendly to render them as form input fields.
        st.text_input(
            label="examples",
            placeholder="Please edit examples with the JSON editor.",
            disabled=True,
            help=(
                "A list of examples to illustrate how the chatbot should respond to "
                "a user."
            ),
        )
        # Upon submitting the form, we will save the form values in to the current
        # prompt config, then update the bot. Any errors should be saved.
        submitted = st.form_submit_button("Update")
        if submitted:
            try:
                # Strings need to be unescaped e.g. from \\n to \n
                current_config = st.session_state.bot.prompt.to_dict()
                current_config["preamble"] = utils.unescape_string(preamble)
                current_config["example_separator"] = utils.unescape_string(
                    example_separator
                )
                current_config["headers"]["user"] = utils.unescape_string(user_name)
                current_config["headers"]["bot"] = utils.unescape_string(bot_name)
                st.session_state.bot.prompt.update(current_config)
                st.session_state.error = ""
            except Exception as e:
                st.session_state.error = e


def draw_prompt_json_editor(max_height: int) -> None:
    """Renders an streamlit-ace editor into the app.

    streamlit-ace is a custom Streamlitcomponent for an Ace editor.
    Reference: https://github.com/okld/streamlit-ace

    Args:
        max_height (int): Desired height of the UI element expressed in pixels.
            If set to None, height will auto adjust to editor's content.
            None by default.
    """
    st.write("**Prompt (JSON):**")
    st_ace(
        value=f"{st.session_state.bot.prompt.to_json_string()}",
        placeholder="Enter a JSON representation of a prompt.",
        height=max_height,
        language="json",
        wrap=True,
        auto_update=True,
        key="json_editor_input",
        theme="monokai",
    )


def draw_prompt_view(json: bool = False) -> None:
    """Adds a representation of the prompt in JSON or as a string.

    Args:
        json (bool): Whether to render the prompt as a JSON object.
    """
    if json:
        st.write("**Prompt (JSON):**")
        st.json(st.session_state.bot.prompt.to_dict())
    else:
        st.write(
            f"**{st.session_state.bot.prompt.bot_name} responds to you using the "
            "prompt below:**"
        )
        # If the current JSON string is malformed, show the error to the user to help
        # with debugging.
        if "error" in st.session_state and st.session_state.error:
            st.exception(st.session_state.error)
        else:
            st.code(
                st.session_state.bot.get_current_prompt("{Your message here}"),
                language="markdown",
            )
            if st.session_state.bot.chat_history:
                st.write("_(includes the current chat history)_")



# Copyright (c) 2022 Cohere Inc. and its affiliates.
#
# Licensed under the MIT License (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License in the LICENSE file at the top
# level of this repository.

import codecs
import os
import pickle
import sys
from typing import Type

import cohere
from streamlit.web import cli as stcli

import conversant
from conversant.chatbot import Chatbot


def encode_chatbot(chatbot: Type[Chatbot]) -> str:
    """Serialize and encode a Chatbot object to a base-64 string encoding.

    Args:
        chatbot (object): a chatbot of class inherited from Chatbot

    Returns:
        str: Chatbot object as a base-64 string
    """
    chatbot.co = None
    return codecs.encode(pickle.dumps(chatbot), "base64").decode()


def decode_chatbot(chatbot_string: str, client: cohere.Client) -> Type[Chatbot]:
    """Decode and deserialize a Chatbot object.

    Args:
        obj_string (str): a base-64 string encoding

    Returns:
        Type[Chatbot]: a chatbot of class inherited rom Chatbot
    """
    chatbot = pickle.loads(codecs.decode(chatbot_string.encode(), "base64"))
    chatbot.co = client
    return chatbot


def launch_streamlit(chatbot: Type[Chatbot]) -> None:
    """Launches a demo of a chatbot using Streamlit.

    The bot will be a persona available for chatting using the interface
    defined in conversant/demo/streamlit_example.py.

    Args:
        bot (Type[Chatbot]): a chatbot of class inherited from Chatbot
    """
    path = os.path.dirname(conversant.__file__)
    sys.argv = f"streamlit run {path}/demo/streamlit_example.py --".split(" ")
    sys.argv.append(encode_chatbot(chatbot))
    sys.exit(stcli.main())



# Copyright (c) 2022 Cohere Inc. and its affiliates.
#
# Licensed under the MIT License (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License in the LICENSE file at the top
# level of this repository.


import ast
import copy
import os
import sys

import cohere
import emoji
import streamlit as st

from conversant.demo import ui, utils
from conversant.prompt_chatbot import PERSONA_MODEL_DIRECTORY, PromptChatbot
from conversant.utils import demo_utils

# Set a custom persona directory by changing the following line
# e.g. "/Users/yourname/custom-personas"
# If the line is left as `CUSTOM_PERSONA_DIRECTORY = None`
# the Streamlit app will use the demo presets
CUSTOM_PERSONA_DIRECTORY = None
USER_AVATAR_SHORTCODE = ":bust_in_silhouette:"


def get_reply() -> None:
    """Replies query from the message input, and resets the message input"""
    _ = st.session_state.bot.reply(query=st.session_state.message_input)
    st.session_state.message_input = ""


def initialize_chatbot() -> None:
    """Initializes the chatbot from a selected persona and saves the session state."""
    if st.session_state.persona.startswith("(launched)") and len(sys.argv) > 1:
        st.session_state.bot = demo_utils.decode_chatbot(
            sys.argv[1], client=cohere.Client(os.environ.get("COHERE_API_KEY"))
        )  # Launched via demo_utils.launch_streamlit() utility function
    elif st.session_state.persona == "":
        st.session_state.bot = None
    elif st.session_state.persona == "parrot":
        st.session_state.bot = utils.ParrotChatbot()
    else:
        st.session_state.bot = PromptChatbot.from_persona(
            emoji.replace_emoji(st.session_state.persona, "").strip(),
            client=cohere.Client(os.environ.get("COHERE_API_KEY")),
            persona_dir=CUSTOM_PERSONA_DIRECTORY
            if CUSTOM_PERSONA_DIRECTORY
            else PERSONA_MODEL_DIRECTORY,
        )
    if "bot" in st.session_state and st.session_state.bot:
        update_session_with_prompt()
    # Reset the edit_promp_json session state so we don't remain on the JSON editor when
    # changing to another bot. This is because st_ace is unable to write
    # new values from the current session state.
    st.session_state.edit_prompt_json = False


def update_session_with_prompt() -> None:
    """Saves the prompt config dictionary into the session state."""
    if "bot" in st.session_state and st.session_state.bot:
        st.session_state.snapshot_prompt_config = copy.deepcopy(
            st.session_state.bot.prompt.to_dict()
        )
        st.session_state.snapshot_chatbot_config = copy.deepcopy(
            st.session_state.bot.chatbot_config
        )
        st.session_state.snapshot_client_config = copy.deepcopy(
            st.session_state.bot.client_config
        )
        st.session_state.current_stop_sequences = [
            utils.escape_string(stop_seq)
            for stop_seq in st.session_state.bot.client_config["stop_sequences"]
        ]


def update_prompt_from_json() -> None:
    """Evaluates JSON string and updates the session's bot prompt."""
    if st.session_state.json_editor_input:
        try:
            prompt_config = ast.literal_eval(st.session_state.json_editor_input)
            st.session_state.bot.prompt.update(prompt_config)
            update_session_with_prompt()
            st.session_state.error = ""
        except Exception as e:
            st.session_state.error = e


# This ensures rendering is prevented upon import of this file.
if __name__ == "__main__":
    st.set_page_config(
        page_title="Conversational personas using Cohere",
        page_icon="🎭",
        layout="wide",
    )

    # Streamlit's default elements are not easy to style. Instead, we have to
    # define styling in a custom CSS file and inject it into the Streamlit DOM.
    # This is brittle and dependent on the DOM structure. Any changes to the layout
    # will break the styling defined in this file.
    with open(f"{os.path.dirname(__file__)}/styles.css") as f:
        utils.style_using_css(f.read())

    # We use the :bust_in_silhouette: emoji as a neutral user avatar.
    st.session_state.user_avatar = utils.get_twemoji_url_from_shortcode(
        USER_AVATAR_SHORTCODE
    )

    # Each persona is a directory in PERSONA_MODEL_DIRECTORY, each with its
    # config.json file.
    if CUSTOM_PERSONA_DIRECTORY:
        st.session_state.persona_options = utils.get_persona_options(
            CUSTOM_PERSONA_DIRECTORY
        )
    else:
        st.session_state.persona_options = utils.get_persona_options()

    # Check if COHERE_API_KEY is not set from secrets.toml or os.environ
    if "COHERE_API_KEY" not in os.environ:
        raise KeyError(
            "COHERE_API_KEY not found in st.secrets or os.environ. Please set it in "
            ".streamlit/secrets.toml or as an environment variable."
        )

    # A chatbot can be passed in as a base64 encoding of a pickled PromptChatbot object.
    # This is only used when calling the launch_demo() method of a PromptChatbot object.
    # The chatbot is then injected into the list of available personas in this streamlit
    # demo.
    if len(sys.argv) > 1 and "bot" not in st.session_state:

        # The PromptChatbot passed in should be a base64 encoding of a pickled
        # PromptChatbot object.
        bot = demo_utils.decode_chatbot(
            sys.argv[1], cohere.Client(os.environ.get("COHERE_API_KEY"))
        )
        if not isinstance(bot, PromptChatbot):
            raise TypeError("base64 string passed in is not of class PromptChatbot")
        else:
            st.session_state.bot = bot
            st.session_state.persona_options.insert(
                0, f"(launched) {st.session_state.bot.persona_name}"
            )

    # Adding a header to direct users to sign up for Cohere, explore the playground,
    # and check out our git repo.
    st.header("🎭 Conversational Personas using Cohere")
    with st.expander("About", expanded="bot" not in st.session_state):
        st.markdown(
            """
        This demo app is using 
        [**conversant**](https://github.com/cohere-ai/sandbox-conversant-lib), an 
        open-source framework for building chatbots on top of Cohere’s large 
        language models. 

        Cohere provides access to advanced Large Language Models and NLP tools through 
        one easy-to-use API. 
        """
            "[**Get started for free!**]"
            "(https://dashboard.cohere.ai/welcome/register?utm_source=cohere-owned&utm_"
            "medium=content&utm_campaign=sandbox&utm_term=streamlit&utm_content=conversant)"
        )

    # Page control flow logic is determined from the sidebar.
    with st.sidebar:
        st.selectbox(
            "Choose a chatbot persona:",
            options=st.session_state.persona_options,
            key="persona",
            on_change=initialize_chatbot,
        )
        st.checkbox(
            "Edit prompt",
            value=False,
            key="edit_prompt",
            on_change=update_session_with_prompt,
        )
        if st.session_state.edit_prompt:
            st.checkbox(
                "Use JSON editor",
                value=False,
                key="edit_prompt_json",
                on_change=update_session_with_prompt,
            )

        # Initialize a settings container in the sidebar. This allows us to place
        # Streamlit elements within this placeholder later in this script.
        settings_placeholder = st.empty()

    # Initialize a chat container as the middle of 3 vertical columns.
    # Only visible when the edit prompt checkbox is not selected.
    _, chat_placeholder, _ = st.columns([1, 1, 1])
    with chat_placeholder.container():
        chat_history_placeholder = st.empty()
        message_input_placeholder = st.empty()

    # Initialize a prompt json and string view as 2 vertical columns.
    # Only visible when the edit prompt checkbox is selected.
    prompt_json_column, prompt_string_column = st.columns([1, 1])
    with prompt_json_column:
        prompt_json_edit_placeholder = st.empty()
        prompt_json_view_placeholder = st.empty()
    with prompt_string_column:
        prompt_string_placeholder = st.empty()

    # Check if bot has been initialized in the Streamlit session.
    if "bot" in st.session_state and st.session_state.bot:

        # Initialize the bot avatar
        bot_avatar_string = st.session_state.bot.chatbot_config["avatar"]
        st.session_state.bot_avatar = (
            utils.get_twemoji_url_from_shortcode(bot_avatar_string)
            if emoji.is_emoji(emoji.emojize(bot_avatar_string, language="alias"))
            else bot_avatar_string
        )

        # Editor view for the prompt
        if st.session_state.edit_prompt:

            # Edit the prompt using a JSON editor
            if st.session_state.edit_prompt_json:

                # The prompt JSON editor needs to be drawn first so that
                # the displayed form values in the sidebar take reference from
                # the editor.
                with prompt_json_edit_placeholder.container():
                    ui.draw_prompt_json_editor(
                        max_height=955
                    )  # st_ace only accepts hardcoded pixel values
                    update_prompt_from_json()

                with settings_placeholder.container():
                    with st.expander("Client Config"):
                        ui.draw_client_config_form()
                    with st.expander("Chatbot Config"):
                        ui.draw_chatbot_config_form()
                    ui.draw_prompt_form(disabled=True)

                with prompt_string_placeholder.container():
                    ui.draw_prompt_view(json=False)

            # Edit the prompt using a form in the sidebar
            else:

                # The settings form needs to be drawn first so that
                # the displayed JSON values in prompt JSON placeholder
                # take reference from the form.
                with settings_placeholder.container():
                    with st.expander("Client Config"):
                        ui.draw_client_config_form()
                    with st.expander("Chatbot Config"):
                        ui.draw_chatbot_config_form()
                    ui.draw_prompt_form(disabled=False)

                with prompt_json_view_placeholder.container():
                    ui.draw_prompt_view(json=True)

                with prompt_string_placeholder.container():
                    ui.draw_prompt_view(json=False)

        # Chat view with the persona
        else:

            # We can get the chatbot to begin the conversation with this.
            # The session's state needs to be manually updated since we are not
            # refreshing the entire Streamlit app.
            if not st.session_state.bot.chat_history:
                st.session_state.bot.reply(
                    query="Hello",
                )
                update_session_with_prompt()

            # Draw UI elements for the sidebar
            with settings_placeholder.container():

                with st.expander("Client Config"):
                    ui.draw_client_config_form()
                with st.expander("Chatbot Config"):
                    ui.draw_chatbot_config_form()

                with st.expander("Prompt (JSON)"):
                    ui.draw_prompt_view(json=True)

                with st.expander("Prompt (string)", expanded=True):
                    ui.draw_prompt_view(json=False)

            # Draw chat history.
            with chat_history_placeholder.container():
                ui.draw_chat_history()

            # Draw the message input field and a disclaimer.
            with message_input_placeholder.container():
                st.text_input(
                    label=f"Chat with {st.session_state.bot.prompt.bot_name}!",
                    placeholder="Type a message",
                    key="message_input",
                    on_change=get_reply,
                )
                ui.draw_disclaimer()

            # When in chat view, anchor elements from the bottom so that
            # the message input field is at the bottom (more natural).
            utils.style_using_css(
                """div.css-18e3th9.egzxvld2 > div:nth-child(1) > div:nth-child(1) > div:nth-child(6) { /* # noqa */
                    margin-top: auto;
                }
            """
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['conversant',
 'conversant.demo',
 'conversant.prompts',
 'conversant.search',
 'conversant.utils']

package_data = \
{'': ['*'],
 'conversant': ['personas/client-support/*',
                'personas/fantasy-wizard/*',
                'personas/fortune-teller/*',
                'personas/historian/*',
                'personas/injured-person/*',
                'personas/math-teacher/*',
                'personas/personal-trainer/*',
                'personas/travel-advisor/*',
                'personas/watch-sales-agent/*']}

install_requires = \
['cohere>=2.8.0,<3.0.0',
 'emoji>=2.1.0,<3.0.0',
 'emojificate>=0.6.0,<0.7.0',
 'pydantic>=1.10.2,<2.0.0',
 'streamlit-ace>=0.1.1,<0.2.0',
 'streamlit-talk>=0.0.4,<0.0.5',
 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'conversant',
    'version': '0.1.7',
    'description': 'Conversational AI tooling',
    'long_description': '```\n################################################################################\n#    ____      _                     ____                  _ _                 #\n#   / ___|___ | |__   ___ _ __ ___  / ___|  __ _ _ __   __| | |__   _____  __  #\n#  | |   / _ \\| \'_ \\ / _ \\ \'__/ _ \\ \\___ \\ / _` | \'_ \\ / _` | \'_ \\ / _ \\ \\/ /  #\n#  | |__| (_) | | | |  __/ | |  __/  ___) | (_| | | | | (_| | |_) | (_) >  <   #\n#   \\____\\___/|_| |_|\\___|_|  \\___| |____/ \\__,_|_| |_|\\__,_|_.__/ \\___/_/\\_\\  #\n#                                                                              #\n# This project is part of Cohere Sandbox, Cohere\'s Experimental Open Source    #\n# offering. This project provides a library, tooling, or demo making use of    #\n# the Cohere Platform. You should expect (self-)documented, high quality code  #\n# but be warned that this is EXPERIMENTAL. Therefore, also expect rough edges, #\n# non-backwards compatible changes, or potential changes in functionality as   #\n# the library, tool, or demo evolves. Please consider referencing a specific   #\n# git commit or version if depending upon the project in any mission-critical  #\n# code as part of your own projects.                                           #\n#                                                                              #\n# Please don\'t hesitate to raise issues or submit pull requests, and thanks    #\n# for checking out this project!                                               #\n#                                                                              #\n################################################################################\n```\n\n**Maintainer:** [Cohere ConvAI Team](mailto:convai@cohere.com) \\\n**Project maintained until at least (YYYY-MM-DD):** 2023-03-01\n\n# Conversant\n[![tests](https://github.com/cohere-ai/sandbox-conversant-lib/actions/workflows/run_tests.yaml/badge.svg)](https://github.com/cohere-ai/sandbox-conversant-lib/actions/workflows/run_tests.yaml/badge.svg)\n[![PyPI](https://img.shields.io/pypi/v/conversant.svg)](https://img.shields.io/pypi/v/conversant.svg)\n[![License: MIT](https://img.shields.io/badge/License-MIT-green.svg)](https://opensource.org/licenses/MIT)\n\nBuild conversational AI on top of [Cohere](https://cohere.ai/)\'s [large language models](https://docs.cohere.ai/generate-reference/)\n- 🗣 Use large language models quickly with Cohere\'s [API](https://docs.cohere.ai/api-reference/)\n- 😃 Customize personas\n- 💁 Leave management of chat history up to `conversant`\n- 🪨 Ground conversations in your provided facts (soon!)\n- 🐍 Install `conversant` with pip\n\n`conversant` is a work-in-progress framework for building customizable dialogue agents (aka chatbots) that can answer questions and converse with users with a variety of different chatbot personas. `conversant` aims\nto be modular, flexible and extensible so you can create any kind of chatbots you want!\n\nWe provide several custom personas for you, including 🧑\u200d💼 a client support agent, ⌚️ a watch sales agent, 🧑\u200d🏫 a math teacher, and 🧙 a fantasy wizard. Create your own persona with just a description and some example conversations!\n\nRead more about how `conversant` is part of the Cohere Sandbox on our [launch blog post](https://txt.cohere.ai/introducing-sandbox-coheres-experimental-open-source-initiative/).\n\nTry `conversant` on our Streamlit demo [here](https://conversant.streamlit.app/)! 🎉\n\n## Table of Contents\n1. [Installation and Usage](#installation-and-usage)\n   1. [Installation](#installation)\n   2. [Streamlit Demo](#streamlit-demo)\n   3. [Running Your Own Streamlit Demo](#running-your-own-streamlit-demo)\n   4. [Creating a Custom Persona](#creating-a-custom-persona)\n   5. [Editing a Persona on the Demo](#editing-a-persona-on-the-demo)\n   6. [Usage](#usage)\n2. [How Conversant Works](#how-conversant-works)\n3. [Documentation](#documentation)\n4. [Get Support](#get-support)\n5. [Contributing Guidelines](#contributing-guidelines)\n6. [License](#license)\n\n## Installation and Usage\n\n### Installation\n\n`conversant` is available [on PyPI](https://pypi.org/project/conversant/), and is tested on Python 3.8+ and [Cohere](https://pypi.org/project/cohere/) 2.8.0+.\n```\npip install conversant\n```\n### Streamlit Demo\n\nWant to see it in action first? You can use `conversant` on a [Streamlit](https://docs.streamlit.io/) app without installing anything [here](https://conversant.streamlit.app/)! 🎉\n\n<p float="none">\n  <img src="https://github.com/cohere-ai/sandbox-conversant-lib/raw/main/static/fortune-teller-setup.png" alt="Screenshot showing the available personas on the Streamlit demo, with the Fortune Teller persona selected by default.." height="550"/>\n  <img src="https://github.com/cohere-ai/sandbox-conversant-lib/raw/main/static/fortune-teller-chat.png" alt="Screenshot showing an exchange between a Fortune Teller chatbot and a user." height="550"/>\n</p>\n\n### Running Your Own Streamlit Demo\n\nCohere uses Streamlit to create its demo applications. If you’re new to Streamlit, you can install it [here](https://docs.streamlit.io/library/get-started/installation) and read more about running Streamlit commands [here](https://docs.streamlit.io/library/get-started/main-concepts).\n\nIf you would like to modify this Streamlit demo locally, we strongly recommend forking this repository rather than installing it as a library from PyPI.\n\nIf you\'d like to spin up your own instance of the Streamlit demo, you will first need a `COHERE_API_KEY`. \nYou can generate one by visiting [dashboard.cohere.ai](https://dashboard.cohere.ai/welcome/register?utm_source=github&utm_medium=content&utm_campaign=sandbox&utm_content=conversant). \n\n#### Local Streamlit apps\nIf you plan to run the Streamlit app locally, you can add the key to `.streamlit/secrets.toml`:\n```\nCOHERE_API_KEY = "YOUR_API_KEY_HERE"\n```\n\nWhen running locally, Streamlit will read the `secrets.toml` file and silently inject these values into the environment variables. Alternatively, you may directly set the API key as an environment variable by running the following command from the command line:\n```\nexport COHERE_API_KEY = "YOUR_API_KEY_HERE"\n```\n\nStart the Streamlit app from the command line with the following command:\n```\nstreamlit run conversant/demo/streamlit_example.py\n```\n\n#### Hosted Streamlit apps\nIf instead you would like to create a hosted Streamlit app, add your Cohere API key to Streamlit via [Secrets Management](https://docs.streamlit.io/streamlit-cloud/get-started/deploy-an-app/connect-to-data-sources/secrets-management). Add the following line as a Secret:\n```\nCOHERE_API_KEY = "YOUR_API_KEY_HERE"\n```\n\n### Creating a Custom Persona\nOnce you have your own instance of the Streamlit app, you can begin experimenting with creating custom personas! Check out the `config.json` for each persona in [`conversant/personas`](https://github.com/cohere-ai/sandbox-conversant-lib/tree/main/conversant/personas) directory. You\'ll need to create a subfolder within this directory that corresponds to your new persona and add a `config.json` file. \n\nAs a note, we strongly recommend forking the `sandbox-conversant-lib` repository rather than installing it as a library from PyPI. When you create a new persona, use the `personas` directory in the cloned repository. The directory structure should look like this:\n\n```\nconversant/personas\n├── fortune-teller\n│   └── config.json\n└── your-persona-name # new\n    └── config.json\n```\n\nThe config file should contain the following:\n- `chatbot_config`: \n  -  `max_context_examples`: The length of the chat history for the chatbot to use in reply.\n  -  `avatar`: Optional emoji shortcode or URL to image as the chatbot\'s avatar. Defaults to 🤖.\n-  `client_config`: Parameters for [`co.generate()`](https://docs.cohere.ai/generate-reference)\n-  `chat_prompt_config`: \n   - `preamble`: Description of the persona.\n   - `example_separator`: A string that separates each example conversation.\n   - `headers`: A name for the `bot` and the `user`.\n   - `examples`: A few conversation examples (few-shot), or empty (zero-shot).\n\n`conversant` will take care of the rest! As an example, check out [`fortune-teller/config.json`](https://github.com/cohere-ai/sandbox-conversant-lib/blob/main/conversant/personas/fortune-teller/config.json). When you launch the Streamlit app, the new persona will appear in the drop down menu.\n\n#### Running the app with a subset of custom personas\n\nIf you would like to run the app with a subset of custom personas, it\'s possible to create a new directory that contains only the desired ones. This is analogous to the `conversant/personas` directory, and needs to have the same structure:\n```\ncustom-personas\n├── your-first-persona\n│   └── config.json\n└── your-second-persona\n    └── config.json\n```  \n\nAfter creating this directory, you\'ll need to tell the app where to look for it. In the demo Streamlit app (`streamlit_example.py`), one of the \nfirst lines reads `CUSTOM_PERSONA_DIRECTORY = None`. Change this to specify the desired \npersona directory, e.g. `CUSTOM_PERSONA_DIRECTORY = "/Users/yourname/custom-personas"`.\n\nIf this is unchanged, the app will default to using the directory that contains the \n`conversant` demo personas.\n\n#### Troubleshooting missing personas\n\nIf you do not see the new persona in the drop down menu, you may need to specify a \ncustom persona directory. Follow [the instructions above](#running-the-app-with-a-subset-of-custom-personas) to tell the app where to look for the personas.\n\n### Editing a Persona on the Demo\nYou can also edit a persona on the Streamlit app!\n<img src="https://github.com/cohere-ai/sandbox-conversant-lib/raw/main/static/fortune-teller-edit.png" alt="Screenshot showing the interface for editing a persona on the Streamlit app."/>\n\n### Usage\n\nWith `conversant`, you can create a chatbot powered by [Cohere](https://cohere.ai/)\'s large language models with just the following code snippet.\n```python\nimport cohere\n\nco = cohere.Client("YOUR_API_KEY_HERE")\nbot = conversant.PromptChatbot.from_persona("fantasy-wizard", client=co)\nprint(bot.reply("Hello!"))\n>>> "Well met, fair traveller. What bringest thou to mine village?"\n```\n\nYou can also define your own persona by passing in your own `ChatPrompt`. \n```python\nfrom conversant.prompts import ChatPrompt\n\nshakespeare_config = {\n    "preamble": "Below is a conversation between Shakespeare and a Literature Student.",\n    "example_separator": "<CONVERSATION>\\n",\n    "headers": {\n        "user": "Literature Student",\n        "bot": "William Shakespeare",\n    },\n    "examples": [\n        [\n            {\n                "user": "Who are you?",\n                "bot": "Mine own nameth is Shakespeare, and I speaketh in riddles.",\n            },\n        ]\n    ],\n}\nshakespeare_bot = PromptChatbot(\n    client=co, prompt=ChatPrompt.from_dict(shakespeare_config)\n)\nprint(shakespeare_bot.reply("Hello!"))\n>>> "Greeteth, and welcome. I am Shakespeare, the great poet, dramatist, and playwright."\n```\n\n<!-- From here, it\'s also possible to talk to your chatbot using the [Streamlit](https://docs.streamlit.io/) app! This will launch the demo with your chatbot persona pre-selected. For this to work, `COHERE_API_KEY` needs to be set as an environment variable.\n```python\nfrom conversant.utils import demo_utils\ndemo_utils.launch_streamlit(shakespeare_bot)\n``` -->\n## How Conversant Works\n`conversant` uses prompt completion to define a chatbot persona with a description and a few examples. The prompt is sent as input to Cohere\'s [`co.generate()`](https://docs.cohere.ai/generate-reference/) endpoint for an autoregressive language model to generate text in a few-shot manner from the examples and the current dialogue context. \n\nEach user message and chatbot response is appended to a chat history so that future responses are conditioned on the dialogue context at that point in time.\n\nIn the future, we plan to add functionality for a chatbot to be factually grounded using text that is retrieved from a local document cache.\n\nFor more information, refer to [this section in `CONTRIBUTORS.md`](https://github.com/cohere-ai/sandbox-conversant-lib/blob/main/CONTRIBUTORS.md#conversant-schematic).\n\n## Documentation\nFull documentation can be found [here](https://cohere-ai.github.io/sandbox-conversant-lib/).\n\n## Get Support\n\nIf you have any questions or comments, please file an issue or reach out to us on [Discord](https://discord.gg/co-mmunity).\n\n## Contributing Guidelines\nIf you would like to contribute to this project, please read [`CONTRIBUTORS.md`](https://github.com/cohere-ai/sandbox-conversant-lib/blob/main/CONTRIBUTORS.md)\nin this repository, and sign the Contributor License Agreement before submitting\nany pull requests. A link to sign the Cohere CLA will be generated the first time \nyou make a pull request to a Cohere repository.\n\nIn addition to guidelines around submitting code to this repository, [`CONTRIBUTORS.md`](https://github.com/cohere-ai/sandbox-conversant-lib/blob/main/CONTRIBUTORS.md) contains a walkthrough to help developers get started, as well as schematics that explain how `conversant` works under the hood. :wrench:\n\n## License\n`conversant` has an [MIT License](https://github.com/cohere-ai/sandbox-conversant-lib/blob/main/LICENSE).\n',
    'author': 'Cohere ConvAI',
    'author_email': 'convai@cohere.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/cohere-ai/sandbox-conversant-lib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
